
// MPLABX IDE 6.0
// MPLAB X XC8XC8 V2.32
//PIC18F-K_DFP 1.5.114
 /*
 Pin 28  RB7  --x  x----ICSP Data
 Pin 27  RB6  --x  x-----ICSP Clock
 Pin 26  RB5   
 Pin 25  RB4   
 Pin 24  RB3   
 Pin 23  RB2   
 Pin 22  RB1  
 Pin 21  RB0   

 Pin 20  VDD --------+5V  ---------------------------------ou-------HC05 Bcc
 Pin 19  Vss -------Gnd--- fil noir cable prolific  ------ ou ------HC05 Gnd
 Pin 18  RC7  RX UART  <-- fil Vert prolific Keyboard  Terminal  ou HC05  TX
 Pin 17  RC6  TX UART  --> fil Blanc prolific Display  Terminal  ou HC05 RX via 1k/2,7k
 Pin 16  RC5  -----------> fil Bleu Prolific connecteur Violet #5
 Pin 15  RC4 
 Pin 14  RC3 
 Pin 13  RC2 
 Pin 12  RC1 ---> PWM CCP1 output --> filtre passe bas -> entree  carte SON PC ou ampli BF
 Pin 11  RC0 

 Pin 10  RA6 
 Pin  9  RA7  
 Pin  8  VSS ---------  Gnd
 Pin  7  RA5 
 Pin  6  RA4 ------->  R+ Led 
 Pin  5  RA3 -------> SQA analyser 
 Pin  4  RA2  
 Pin  3  RA1 
 Pin  2  RA0 
 Pin  1  RE3 MCLR  Reset <--------ICSP PGRM
                         |----x  x----/0V   cavalier RESET
 */
#define VERSION "2024-126"
// avec datas wav sur choix de fichier include *.txt 

//#define VERSION "2022-0922"
// version OK avec 1 ou 2  tables tailles definie dans l'init  64000 et 42304
// voir https://www.fantaspic.fr/viewtopic.php?f=10&t=1246&p=18661#p18661

#include <xc.h>
#include <pic18f27k42.h>
#include <stdio.h>
#include <stdint.h>

#define _XTAL_FREQ 64000000
#define FOSC       64000000              // Fosc=16MHz
#define FCY       (FOSC/4)              // Fcy=4MHz
#define BR        115200                // Baud Rate  UART Speed !!!!
#define BRG       ((FCY/BR)-1)          // brgs=1

#pragma warning disable 1498
#pragma warning disable 31
#pragma warning disable 228   // caractere francais accentu
#pragma warning disable 520  
#pragma warning disable 1510 
#pragma warning disable 759
#pragma warning disable  2053
#pragma warning disable  2020
#pragma warning disable 359
#pragma warning disable 373

//========================================================
//  manual config bits

 // CONFIG1L
#pragma config FEXTOSC = OFF    // External Oscillator Selection->Oscillator not enabled
#pragma config RSTOSC = HFINTOSC_64MHZ    // Reset Oscillator Selection->HFINTOSC with HFFRQ = 64 MHz and CDIV = 1:1

// CONFIG1H
#pragma config CLKOUTEN = OFF    // Clock out Enable bit->CLKOUT function is disabled
#pragma config PR1WAY = ON    // PRLOCKED One-Way Set Enable bit->PRLOCK bit can be cleared and set only once
#pragma config CSWEN = ON    // Clock Switch Enable bit->Writing to NOSC and NDIV is allowed
#pragma config FCMEN = ON    // Fail-Safe Clock Monitor Enable bit->Fail-Safe Clock Monitor enabled

// CONFIG2L
#pragma config MCLRE = EXTMCLR    // MCLR Enable bit->If LVP = 0, MCLR pin is MCLR; If LVP = 1, RE3 pin function is MCLR 
#pragma config PWRTS = PWRT_OFF    // Power-up timer selection bits->PWRT is disabled
#pragma config MVECEN = ON    // Multi-vector enable bit->Multi-vector enabled, Vector table used for interrupts
#pragma config IVT1WAY = OFF    // IVTLOCK bit One-way set enable bit->IVTLOCK bit can be cleared and set repeatedly
#pragma config LPBOREN = OFF    // Low Power BOR Enable bit->ULPBOR disabled
#pragma config BOREN = SBORDIS    // Brown-out Reset Enable bits->Brown-out Reset enabled , SBOREN bit is ignored

// CONFIG2H
#pragma config BORV = VBOR_2P45    // Brown-out Reset Voltage Selection bits->Brown-out Reset Voltage (VBOR) set to 2.45V
#pragma config ZCD = OFF    // ZCD Disable bit->ZCD disabled. ZCD can be enabled by setting the ZCDSEN bit of ZCDCON
#pragma config PPS1WAY = OFF    // PPSLOCK bit One-Way Set Enable bit->PPSLOCK bit can be set and cleared repeatedly (subject to the unlock sequence)
#pragma config STVREN = ON    // Stack Full/Underflow Reset Enable bit->Stack full/underflow will cause Reset
#pragma config DEBUG = OFF    // Debugger Enable bit->Background debugger disabled
#pragma config XINST = OFF    // Extended Instruction Set Enable bit->Extended Instruction Set and Indexed Addressing Mode disabled

// CONFIG3L
#pragma config WDTCPS = WDTCPS_31    // WDT Period selection bits->Divider ratio 1:65536; software control of WDTPS
#pragma config WDTE = OFF    // WDT operating mode->WDT Disabled; SWDTEN is ignored

// CONFIG3H
#pragma config WDTCWS = WDTCWS_7    // WDT Window Select bits->window always open (100%); software control; keyed access not required
#pragma config WDTCCS = SC    // WDT input clock selector->Software Control

// CONFIG4L
#pragma config BBSIZE = BBSIZE_512    // Boot Block Size selection bits->Boot Block size is 512 words
#pragma config BBEN = OFF    // Boot Block enable bit->Boot block disabled
#pragma config SAFEN = OFF    // Storage Area Flash enable bit->SAF disabled
#pragma config WRTAPP = OFF    // Application Block write protection bit->Application Block not write protected

// CONFIG4H
#pragma config WRTB = OFF    // Configuration Register Write Protection bit->Configuration registers (300000-30000Bh) not write-protected
#pragma config WRTC = OFF    // Boot Block Write Protection bit->Boot Block (000000-0007FFh) not write-protected
#pragma config WRTD = OFF    // Data EEPROM Write Protection bit->Data EEPROM not write-protected
#pragma config WRTSAF = OFF    // SAF Write protection bit->SAF not Write Protected
#pragma config LVP = ON    // Low Voltage Programming Enable bit->Low voltage programming enabled. MCLR/VPP pin function is MCLR. MCLRE configuration bit is ignored

// CONFIG5L
#pragma config CP = OFF    // PFM and Data EEPROM Code Protection bit->PFM and Data EEPROM code protection disabled
//===========================================================


#ifndef Byte
#define Byte  uint8_t
#endif
#ifndef Word
#define Word  uint16_t
#endif

#define ON 1
#define OFF 0
#define SPECIAL 1
#define NORMAL 0
#define VRAI 1
#define FAUX 0
// pour report Alarme   (bit inverse par transistor de sortie)
#define OUI 0
#define NON 1

//#include "SonWav_bourrepif_1_table_64Ko.txt"
#include "SonWav_tou_nu_tou_bronze_datas_en_2_tables.txt"  //  106k   4000+2644 lignes de datas  ok

// usage 1ere table avec SonWav[Taille1];  // si <64Ko
// usage 2em tble avec SonWav1[Taille2];] // la suite


#include "UART1_Functions.h"



Word i,j,k,l,m,n;
int Rep;
uint8_t volatile c2,c3,cx,cc;
Byte temp=0;
uint32_t Adresse;

uint8_t CRam1[64];

 
 #ifndef UART1_RX_BUFFER_SIZE
#define  UART1_RX_BUFFER_SIZE 128  
#endif
volatile unsigned char  c1=0;
volatile unsigned char Buffer1[UART1_RX_BUFFER_SIZE];
volatile unsigned int Index1=0;
volatile unsigned int i1=0;
volatile unsigned char Flag_Buffer1;
 

#define RTS             LATCbits.LATC5
#define Affiche_Detail  PORTBbits.RB1
#define SQA             LATAbits.LATA3

uint32_t L0,L1,L2;
uint32_t File_Size;

/*
void  UART1_Init(void); 
void RAZ_UART1_Buffer1(void);
void PrintChar( char txData);
void Print ( char * T1);
void CPrint (const  char *T);
void CRLF1(void);

*/
void Init_Hardware(void);
void Init_CCP1_PWM(void);
void Init_Timer2(void);

void delay_ms(uint16_t n)
//--------------------------------------------------------------------------------------------
{
  uint32_t d=n; d *= FOSC/40000;
  while (d--) {  }
}

void Init_Timer2()
{ 
  T2CON    = 0x00;
  // resolution 9 bits  => maxi 100% duty avec consigne < 720 points
  T2CONbits.CKPS=3;  //3=> div 1/8   
  T2CONbits.OUTPS=0;  //0=> div 1/1 .......  NOT USED with CCP1 ,only as free Timer
  T2PR=180;  // 0.0625 * 2 * 3 * 240 =90.35 S  =>  11065 Hz
  T2TMR=0;
  T2CLK=0x01; //1= FOSC/4    2=FOSC     5=500Khz   
 // REGISTER 22-2: TxRST: TIMER2 EXTERNAL RESET SIGNAL SELECTION REGISTER p337
  T2RST=0x04;  // CCP1OUT
  PIE4bits.TMR2IE=0;
  PIR4bits.TMR2IF=0;
  T2CONbits.T2ON=0;
}

void Init_CCP1_PWM(void)
 {
    CCP1CON=0;
    CCP1CONbits.CCP1FMT =0;  // right alignment
    CCP1CONbits.CCP1MODE3 =1;   //PWM 0b1100';        //   table 22-1  p322
    CCP1CONbits.CCP1MODE2 =1;
    CCP1CONbits.CCP1MODE1 =0;
    CCP1CONbits.CCP1MODE0 =0;
  //  REGISTER 23-2: CCPTMRS0: CCP TIMERS CONTROL REGISTER 0
  //  01 = CCP1 is based off Timer2 in PWM mode
    CCPTMRS0bits.C1TSEL1=0;
    CCPTMRS0bits.C1TSEL0=1;
    CCP1CAP=0;
    CCP1CONbits.EN=1; 
}


void Init_Hardware()
{
    LATA = 0x00;
    TRISA = 0b1000011;
    ANSELA = 0x00;
    WPUA = 0x00;
    ODCONA = 0x00;
    SLRCONA = 0xFF;
    INLVLA = 0xFF;
    SQA=0;
 
    LATB = 0x00;
    TRISB = 0b11001111;
    ANSELB = 0x00;
    RB0PPS=0;
    RB1PPS=0;
    RB2PPS=0;
    RB3PPS=0;
    INT0PPS = 0x08;   //RB0->EXT_INT:INT0;
    WPUB = 0x03;  // pull up sur RB0 et RB1
    ODCONB = 0x00;
    SLRCONB = 0xFF;
    INLVLB = 0xFF;
    
    LATC = 0x00;
    TRISC = 0b10001001;
    ANSELC = 0x00;
    RC0PPS=0;
    RC1PPS=0x09 ;// CCP1 sur RC1  TABLE 17-2: PPS OUTPUT REGISTER DETAILS p283
    RC2PPS=0;
    RC3PPS=0;
    RC4PPS=0;
    RC5PPS=0; // gestion manuelle de RTS
   // RC5PPS=0x15; // TABLE 17-2: PPS OUTPUT REGISTER  p282  0b01 0101 UART1 (RTS) ? B C
#ifdef With_NCO
    // pour NCO
    //RC4PPS=0x26; //RC4->NCO1:NCO1;
#endif
    RC6PPS = 0x13;   //RC6->UART1:TX1;    
    U1RXPPS = 0x17;   //RC7->UART1:RX1; 
    RTS=1;
    ODCONC = 0x00;
    SLRCONC = 0x0F;
    INLVLC = 0xFF;
    WPUC = 0xFF;
   
}    
  

//**********************************************************************


void main(void)
{
   OSCCON1 = 0x60; // NOSC HFINTOSC; NDIV 1;
   OSCCON3 = 0x00;// CSWHOLD may proceed; SOSCPWR Low power; 
   OSCEN = 0x00;  // MFOEN dis; LFOEN dis; ADOEN dis; SOSCEN dis; EXTOEN dis; HFOEN dis; 
   OSCFRQ = 0x08;  // HFFRQ 64_MHz;
   OSCTUNE = 0x00;// TUN 0; 
   delay_ms(500);
    
   //REGISTER 8-1: CLKRCON: REFERENCE CLOCK CONTROL REGISTER  p113
   CLKRCON=0b10010000;
   //REGISTER 8-2: CLKRCLK: CLOCK REFERENCE CLOCK SELECTION MUX
   CLKRCLK=3; // 500Khz

   Init_Hardware();
   CM1CON0=0;
   CM1CON1=0;
   UART1_Init();
   CPrint("Test_Lect_Wav_en_Flash_18F27K42_2024\r\n");
   CPrint(" SonWav_tou_nu_tou_bronze_datas_en_2_tables.txt  106k \r\n");
                 
   k=128;
   // suivant config FORMAT   voir FIGURE 23-5: PWM 10-BIT ALIGNMENT  p349
   CCPR1L  = (k & 0x00FF);      //   Writing  LSB of PWM duty cycle
   CCPR1H = ((k & 0x03FF)>>8) ; //  Writing 2 bits MSB of PWM duty cycle
   // mais ici la data ne depasse pas 255 donc LSB suffisant  MSB=0
   T2CONbits.T2ON=0;
   
   
   
     L1=sizeof(SonWav);
     L2=sizeof(SonWav1);
    sprintf(CRam1,"SonWav size L1=%u , SonWav1 L2=%u \r\n",L1,L2);
    Print(CRam1);
    CRLF1();
     
   L1=sizeof(SonWav);
    sprintf(CRam1,"SonWav L1=%u   ",L1);  //  
    Print(CRam1);
   L2=sizeof(SonWav1);    
   sprintf(CRam1," SonWav1 L2=%u \r\n",L2);  //  
   Print(CRam1);

   INTCON0bits.GIEL =0;
   INTCON0bits.GIEH=0;
   Init_Timer2();
   Init_CCP1_PWM();
   CCP1CONbits.EN=1; 
   T2CONbits.T2ON=1; 
   for (L0=64; L0<L1  ;L0++)   
   {  // T2PR=180;
            T2TMR=0;
            CCPR1H=0;
            CCPR1L  =SonWav[L0];
           while( PIR4bits.TMR2IF==0);
            PIR4bits.TMR2IF=0;       
   }
             
   L0=0;
   for (L0=0; L0<L2  ;L0++)   
   {  // T2PR=180;
            T2TMR=0;
            CCPR1H=0;
            CCPR1L  =SonWav1[L0];
           while( PIR4bits.TMR2IF==0);
            PIR4bits.TMR2IF=0;       
   }
     T2CONbits.T2ON=0;                      
    CCP1CONbits.EN=0;   
    CPrint("\r\nReset\r\n");
    delay_ms(4000);
     __asm("reset"); 

}
  
/*
(15:20:30.212) Test_Lect_Wav_en_Flash_18F27K42_2024
(15:20:30.255)  SonWav_tou_nu_tou_bronze_datas_en_2_tables.txt  106k 
(15:20:30.255) SonWav size L1=64000 , SonWav1 L2=0 
(15:20:30.255) 
(15:20:30.255) SonWav L1=64000    SonWav1 L2=42304 
(15:20:40.086) 
(15:20:40.086) Reset
(15:20:45.027) Test_Lect_Wav_en_Flash_18F27K42_2024
(15:20:45.070)  SonWav_tou_nu_tou_bronze_datas_en_2_tables.txt  106k 
(15:20:45.070) SonWav size L1=64000 , SonWav1 L2=0 
(15:20:45.070) 
(15:20:45.070) SonWav L1=64000    SonWav1 L2=42304 
*/
